@@ -3,16 +3,20 @@ import shlex
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from enum import Enum
-from typing import TypeVar, Callable, Type, ClassVar
+from typing import Any, TypeVar, Callable, Type, ClassVar
 
 T = TypeVar("T", bound="BaseStatus")
 
+if False:  # for mypy
+    from .machine import Machine
 
+
+@dataclass
 class BaseStatus(ABC):
     @classmethod
     @property
     @abstractmethod
-    def _comlist(cls: Type[T]) -> dict[str, tuple[bytes, Callable]]:
+    def _comlist(cls: Type[T]) -> dict[str, tuple[bytes, Callable[[Any], Any]]]:
         ...
 
     @classmethod
@@ -27,7 +31,7 @@ class BaseStatus(ABC):
         return cls.from_bytes(out)
 
     @classmethod
-    def from_bytes(cls: Type[T], out: bytes) -> Type[T]:
+    def from_bytes(cls: Type[T], out: bytes) -> T:
         return cls(
             **{
                 k: inst(v)
@@ -38,7 +42,7 @@ class BaseStatus(ABC):
         )
 
 
-def _get_protodef_or_def(var, default):
+def _get_protodef_or_def(var: str, default: Any):
     return f"$[ top.getChild('PROTOcolDEFinition').variables.get('{var}'.lower(), {default}) ]".encode()
 
 
@@ -53,7 +57,7 @@ class RunStatus(BaseStatus):
     point: int
     state: str
 
-    _comlist: ClassVar[dict[str, tuple[bytes, Callable]]] = {
+    _comlist: ClassVar[dict[str, tuple[bytes, Callable[[Any], Any]]]] = {
         "name": (b"${RunTitle:--}", str),
         "stage": (b"${Stage:--1}", lambda x: int(x) if x != "PRERUN" else 0),
         "num_stages": (_get_protodef_or_def("${RunMacro}-Stages", -1), int),
@@ -75,7 +79,7 @@ class MachineStatus(BaseStatus):
     cover: str
     lamp_status: str
 
-    _comlist: ClassVar[dict[str, tuple[bytes, Callable]]] = {
+    _comlist: ClassVar[dict[str, tuple[bytes, Callable[[Any], Any]]]] = {
         "drawer": (b"$(DRAWER?)", str),
         "cover": (b'$[ "$(ENG?)" or "unknown" ]', str),
         "lamp_status": (b"$(LST?)", str),
@@ -100,25 +104,40 @@ class AccessLevel(Enum):
     Administrator = "Administrator"
     Full = "Full"
 
-    def __gt__(self, other):
-        if self.__class__ == other.__class__:
+    def __gt__(self, other: object) -> bool:
+        if isinstance(other, str):
+            other = AccessLevel(other)
+        if isinstance(other, AccessLevel):
             return _accesslevel_order[self.value] > _accesslevel_order[other.value]
         raise NotImplementedError
 
-    def __ge__(self, other):
-        if self.__class__ == other.__class__:
+    def __ge__(self, other: object) -> bool:
+        if isinstance(other, str):
+            other = AccessLevel(other)
+        if isinstance(other, AccessLevel):
             return _accesslevel_order[self.value] >= _accesslevel_order[other.value]
         raise NotImplementedError
 
-    def __lt__(self, other):
-        if self.__class__ == other.__class__:
+    def __lt__(self, other: object) -> bool:
+        if isinstance(other, str):
+            other = AccessLevel(other)
+        if isinstance(other, AccessLevel):
             return _accesslevel_order[self.value] < _accesslevel_order[other.value]
         raise NotImplementedError
 
-    def __le__(self, other):
-        if self.__class__ == other.__class__:
+    def __le__(self, other: object) -> bool:
+        if isinstance(other, str):
+            other = AccessLevel(other)
+        if isinstance(other, AccessLevel):
             return _accesslevel_order[self.value] <= _accesslevel_order[other.value]
         raise NotImplementedError
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, str):
+            other = AccessLevel(other)
+        if isinstance(other, AccessLevel):
+            return _accesslevel_order[self.value] == _accesslevel_order[other.value]
+        raise NotImplementedError
+
     def __str__(self) -> str:
         return self.value
