@@ -1,5 +1,5 @@
-from .machine import Machine
-from .experiment import Experiment, PlateSetup, Protocol, Stage, Step, RunStatus, MachineStatus
+from .machine import Machine, MachineStatus, RunStatus
+from .experiment import Experiment, PlateSetup, Protocol, Stage, Step
 from . import tcprotocol as tc
 
 __all__ = (
