@@ -1,10 +1,11 @@
 from __future__ import annotations
 import asyncio
+from asyncio.futures import Future
 import logging
 import re
 import io
 from dataclasses import dataclass
-from typing import Coroutine, Optional, Protocol
+from typing import Any, Coroutine, Optional, Protocol
 
 
 NL_OR_Q = re.compile(rb"(?:\n|<(/?)([\w.]+)[ *]*>)")
@@ -32,12 +33,12 @@ class ReplyError(IOError):
 class SubHandler(Protocol):
     def __call__(
         self, topic: bytes, message: bytes, timestamp: float | None = None
-    ) -> Coroutine:
+    ) -> Coroutine[None, None, None]:
         ...
 
 
 class QS_IS_Protocol(asyncio.Protocol):
-    def __init__(self):
+    def __init__(self) -> None:
         self.default_topic_handler = self._default_topic_handler
         self.readymsg = asyncio.get_running_loop().create_future()
         self.lostconnection = asyncio.get_running_loop().create_future()
@@ -51,34 +52,31 @@ class QS_IS_Protocol(asyncio.Protocol):
         self.lostconnection.set_result(exc)
         self.should_be_connected = False
 
-    async def disconnect(self):
+    async def disconnect(self) -> None:
         self.should_be_connected = False
         await self.run_command("QUIT")
 
-    def connection_made(self, transport):
+    def connection_made(self, transport: Any) -> None:
         log.info("Made connection")
         self.should_be_connected = True
         # setup connection.
         self.transport = transport
-        self.messages = []
-        self.waiting_commands = []
+        self.waiting_commands: list[tuple[bytes, None | Future[tuple[bytes, bytes]]]] = []
         self.buffer = io.BytesIO()
-        self.quote_stack = []
-        self.topic_handlers: dict[
-            bytes, SubHandler
-        ] = {}
+        self.quote_stack: list[bytes] = []
+        self.topic_handlers: dict[bytes, SubHandler] = {}
         pass
 
     async def _default_topic_handler(
         self, topic: bytes, message: bytes, timestamp: Optional[float] = None
-    ):
+    ) -> None:
         log.info(f"{topic.decode()} at {timestamp}: {message.decode()}")
 
-    async def handle_sub_message(self, message: bytes):
+    async def handle_sub_message(self, message: bytes) -> None:
         i = message.index(b" ")
         topic = message[0:i]
         if m := TIMESTAMP.match(message, i + 1):
-            timestamp = float(m[1])
+            timestamp: float | None = float(m[1])
             i = m.end()
         else:
             timestamp = None
@@ -97,11 +95,11 @@ class QS_IS_Protocol(asyncio.Protocol):
                     if comfut is not None:
                         comfut.set_result((ds[:ms], ds[ms + len(commref) + 2 :]))
                     else:
-                        log.info(f"{commref} complete: {ds}")
+                        log.info(f"{commref!r} complete: {ds!r}")
                     r = i
                     break
             if r is None:
-                log.error(f"received unexpected command response: {ds}")
+                log.error(f"received unexpected command response: {ds!r}")
             else:
                 del self.waiting_commands[r]
         elif ds.startswith(b"MESSage"):
@@ -109,7 +107,7 @@ class QS_IS_Protocol(asyncio.Protocol):
         elif ds.startswith(b"READy"):
             self.readymsg.set_result(ds.decode())
         else:
-            log.error(f"Unknown message: {ds}")
+            log.error(f"Unknown message: {ds!r}")
 
     def data_received(self, data: bytes) -> None:
         """Process received data packet from instrument, keeping track of quotes. If
@@ -137,7 +135,7 @@ class QS_IS_Protocol(asyncio.Protocol):
                         i = self.quote_stack.index(m[2])
                     except ValueError:
                         raise ValueError(
-                            f"Close quote {m[2]} did not have open"
+                            f"Close quote {m[2]!r} did not have open"
                             f" in stack {self.quote_stack}."
                         ) from None
                     else:
@@ -155,17 +153,17 @@ class QS_IS_Protocol(asyncio.Protocol):
     ) -> bytes:
         if isinstance(comm, str):
             comm = comm.encode()
-        log.debug(f"Running command {comm}")
+        log.debug(f"Running command {comm.decode()}")
         loop = asyncio.get_running_loop()
 
-        comfut = loop.create_future()
+        comfut: Future[tuple[bytes, bytes]] = loop.create_future()
         if uid:
             import random
 
             commref = str(random.randint(1, 2 ** 30)).encode()
             comm = commref + b" " + comm
         self.transport.write((comm + b"\n"))
-        log.debug(f"Sent command {comm}")
+        log.debug(f"Sent command {comm!r}")
         if m := re.match(rb"^(\d+) ", comm):
             commref = m[1]
         else:
@@ -174,7 +172,7 @@ class QS_IS_Protocol(asyncio.Protocol):
 
         await asyncio.wait_for(asyncio.shield(comfut), ack_timeout)
         state, msg = comfut.result()
-        log.debug(f"Received ({state}, {msg})")
+        log.debug(f"Received ({state!r}, {msg!r})")
 
         if state == b"NEXT":
             if just_ack:
@@ -185,7 +183,7 @@ class QS_IS_Protocol(asyncio.Protocol):
                 self.waiting_commands.append((commref, comnext))
                 await comnext
                 state, msg = comnext.result()
-                log.debug(f"Received ({state}, {msg})")
+                log.debug(f"Received ({state!r}, {msg!r})")
 
         if state == b"OK":
             return msg
@@ -194,4 +192,6 @@ class QS_IS_Protocol(asyncio.Protocol):
                 comm.decode(), commref.decode(), msg.decode().rstrip()
             ) from None
         else:
-            raise CommandError(comm.decode(), commref.decode(), state + msg)
+            raise CommandError(
+                comm.decode(), commref.decode(), (state + b" " + msg).decode()
+            )
